from __future__ import annotations

import abc
from functools import total_ordering
from typing import Tuple, final

from plover.engine import StenoEngine

import plover_per_application_state.state.manager as _state_manager


@total_ordering
class WindowTransitionHandler(abc.ABC):
    def __init__(self, engine: StenoEngine, priority=0):
        self._engine = engine
        self._priority = priority

    @abc.abstractmethod
    def handle_transition(self, state_manager: _state_manager.StateManager, engine: StenoEngine,
                          old_details: Tuple[int, Tuple[str, str, str]],
                          new_details: Tuple[int, Tuple[str, str, str]]) -> bool:
        return False

    @final
    def __eq__(self, other: 'WindowTransitionHandler') -> bool:
        return self._priority == other._priority

    @final
    def __gt__(self, other: 'WindowTransitionHandler') -> bool:
        return self._priority > other._priority



from __future__ import annotations

from typing import Tuple

from plover.engine import StenoEngine

import plover_per_application_state.state.manager as _state_manager
from plover_per_application_state.transition_handler.handler import WindowTransitionHandler


class DefaultWindowTransitionHandler(WindowTransitionHandler):
    def handle_transition(self, state_manager: _state_manager.StateManager, engine: StenoEngine,
                          old_details: Tuple[int, Tuple[str, str, str]],
                          new_details: Tuple[int, Tuple[str, str, str]]) -> bool:

        old_handle_hash, (old_app, old_class, old_title) = old_details
        new_handle_hash, (new_app, new_class, new_title) = new_details

        stored_state = state_manager.get_state(new_handle_hash, new_title)
        state_manager.store_state(old_handle_hash, old_title, engine.translator_state)
        engine.translator_state = stored_state

        return True



from __future__ import annotations

from typing import Tuple

from plover.engine import StenoEngine
from plover_application_controls import WindowTracker

from plover_per_application_state.state.manager import StateManager
import plover_per_application_state.transition_handler as handlers


class PerApplicationStateExtension:
    _instance: 'PerApplicationStateExtension' = None

    def __init__(self, engine: StenoEngine) -> None:
        PerApplicationStateExtension._instance = self
        self._on = False
        self._engine = engine

        self._state_manager = StateManager([
            handlers.DefaultWindowTransitionHandler(engine)
        ])
        self._last_details = (hash(None), ("", "", ""))

        WindowTracker.add_callback(True, self.on_window_callback)

    def start(self) -> None:
        self._on = True

    def stop(self) -> None:
        self._on = False

    def on_window_callback(self, _, handle_hash: int, details: Tuple[str, str, str]) -> None:
        new_details = (handle_hash, details)
        if not self._on:
            self._last_details = new_details
            return

        self._state_manager.on_transition(self._engine, self._last_details, new_details)
        self._last_details = new_details

    @staticmethod
    def get_state_manager() -> StateManager | None:
        if PerApplicationStateExtension._instance is None:
            return None
        return PerApplicationStateExtension._instance._state_manager
